/*
* Copyright (c) 2006-2007 Erin Catto http:
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked, and must not be
* misrepresented the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

goog.provide('box2d.PulleyJointDef');

goog.require('box2d.JointDef');



// The pulley joint is connected to two bodies and two fixed ground points.
// The pulley supports a ratio such that:
// length1 + ratio * length2 = constant
// Yes, the force transmitted is scaled by the ratio.
// The pulley also enforces a maximum length limit on both sides. This is
// useful to prevent one side of the pulley hitting the top.
/**
 @constructor
 @extends {box2d.JointDef}
 */
box2d.PulleyJointDef = function() {
  box2d.JointDef.call(this, true);
  // The constructor for b2JointDef
  this.type = box2d.Joint.e_unknownJoint;
  this.userData = null;
  this.body1 = null;
  this.body2 = null;
  //
  // initialize instance variables for references
  this.groundPoint1 = new box2d.Vec2();
  this.groundPoint2 = new box2d.Vec2();
  this.anchorPoint1 = new box2d.Vec2();
  this.anchorPoint2 = new box2d.Vec2();
  //
  this.type = box2d.Joint.e_pulleyJoint;
  this.groundPoint1.Set(-1.0, 1.0);
  this.groundPoint2.Set(1.0, 1.0);
  this.anchorPoint1.Set(-1.0, 0.0);
  this.anchorPoint2.Set(1.0, 0.0);
  this.maxLength1 = 0.5 * box2d.PulleyJoint.b2_minPulleyLength;
  this.maxLength2 = 0.5 * box2d.PulleyJoint.b2_minPulleyLength;
  this.ratio = 1.0;
};
goog.inherits(box2d.PulleyJointDef, box2d.JointDef);
